#[macro_use]
extern crate clap;
use clap::{Arg, App};

use std::process;
use std::sync::mpsc;
use std::thread;
use std::time::{Duration,Instant};

const DATA_SIZE : usize = 1;

type DataType = [f32; DATA_SIZE];

fn main() {
    let app_m = App::new("ping-pong")
        .version("0.1.0")
        .author("Michael O. Distler <distler@uni-mainz.de>")
        .about("send messages back and forth between two threads")
        .arg(Arg::with_name("Cycles")
             .short("c")
             .long("cycles")
             .value_name("num")
             .takes_value(true)
             .required(true)
             .help("number of cycles"))
        .arg(Arg::with_name("Sleep")
             .short("s")
             .long("sleep")
             .value_name("millisec")
             .takes_value(true)
             .help("number of milliseconds"))
        .get_matches();

    println!("pid: {}", process::id());
    let n = value_t!(app_m, "Cycles", usize).unwrap();
    let msec = value_t!(app_m, "Sleep", u64).unwrap_or_else(|_| 0);

    // bidirectional channel
    let (main_tx, child_rx) = mpsc::channel::<Box<DataType>>();
    let (child_tx, main_rx) = mpsc::channel::<Box<DataType>>();

    let child = thread::spawn(move || {
        for mut data in child_rx {
            data[0] += 1.0;
            if msec>0 {
                thread::sleep(Duration::from_millis(msec));
            }
            child_tx.send(data).unwrap();
        }
    });

    let mut data = Box::new([0.; DATA_SIZE]);

    let start = Instant::now();
    for _ in 0..n {
        main_tx.send(data).unwrap();
        data = main_rx.recv().unwrap();
    }
    let finish = Instant::now();

    drop(main_tx);
    child.join().unwrap();

    println!("loop count    : {}", data[0]);
    if n>1 {
        println!("transmit time : {:?}",
                 finish.duration_since(start)/((2*n) as u32));
    }
}
