#[macro_use]
extern crate clap;
use clap::{Arg, App};

use std::process;
use std::sync::mpsc::{Sender, Receiver};
use std::sync::mpsc;
use std::thread;

fn main() {
    let app_m = App::new("ring of threads")
        .version("0.1.0")
        .author(concat!("Michael O. Distler <distler@uni-mainz.de> and\n",
                        "Janine Hilder <janicode@uni-mainz.de>"))
        .about("a complicated way to add integers in arithmetic progression")
        .arg(Arg::with_name("Threads")
             .short("t")
             .long("threads")
             .value_name("num")
             .takes_value(true)
             .required(true)
             .help("number of threads"))
        .get_matches();

    println!("pid: {}", process::id());
    let n = value_t!(app_m, "Threads", usize).unwrap();
    let m_result = value_t!(app_m, "sleep", u64);
    println!("{} threads: expected return value {}", n, n*(n+1)/2);

    // define vectors for send/receive endpoints
    let mut sender = Vec::<Sender<u32>>::with_capacity(n);
    let mut receiver = Vec::<Receiver<u32>>::with_capacity(n);

    // create channels for inter thread communication
    for _ in 0..n {
        let (snd, rcv) = mpsc::channel();
        sender.push(snd);
        receiver.push(rcv);
    }
    receiver.rotate_left(1);

    // channel to report result to main thread
    let (report, messages) = mpsc::sync_channel::<(usize,u32)>(1);

    // start N threads
    for (id,(tx,rx)) in sender.iter().zip(receiver).enumerate() {
        let report = report.clone();
        let tx = tx.clone();
        thread::spawn(move || {
            let mut s = (id+1) as u32;
            let mut sum : u32 = 0;
            for _ in 0..n {
                tx.send(s).unwrap();
                s = rx.recv().unwrap();
                sum += s;
            }
            report.send((id,sum)).unwrap();
        });
    }
    drop(report);

    // rx will return null after tx was droped and all threads terminated
    for msg in messages {
        println!("thread {} returned {}", msg.0, msg.1);
    }
}
